Class: Cosmo::Railtie
- Inherits:
-
Rails::Railtie
- Object
- Rails::Railtie
- Cosmo::Railtie
- Defined in:
- lib/cosmo/railtie.rb
Overview
Rails Railtie — loaded automatically when cosmonats is required inside a Rails application. Ensures the ActiveJob adapter constant is registered before Rails tries to resolve it and autoloads config/cosmo.yml when the file is present.